// Generated by CoffeeScript 2.3.0
(function() {
  var Holidays, Moment, chain, commands, debug, get_list, global_egress, global_ingress, ingress_modules, is_blacklisted, is_suspicious, is_whitelisted, local_egress, local_ingress, max_menu_depth, menu_conference_module, pattern, request, run, serialize,
    indexOf = [].indexOf;

  this.name = 'huge-play:middleware:client:commands';

  debug = (require('tangible'))(this.name);

  Moment = require('moment-timezone');

  Holidays = require('date-holidays');

  request = require('superagent');

  run = require('flat-ornament');

  serialize = require('useful-wind-serialize');

  max_menu_depth = 42;

  // Lists handling
  // ==============

  // `list:` ID manufacture
  // ----------------------

  // Note: list IDs are always `<list-name>@<interesting-number>` where the `interesting-number` is the number for which we want to know whether it is whitelisted, blacklisted, etc., while the `list-name` is a unique name for the given list of interesting-numbers.

  // List ID for an ingress call.
  local_ingress = async function() {
    var interesting_number, list, list_id, pid;
    list = (await this.validate_local_number());
    pid = this.req.header('P-Asserted-Identity');
    interesting_number = pid != null ? url.parse(pid).auth : this.source;
    list_id = `list:${list}@${interesting_number}`;
    this.debug('local_ingress', list, interesting_number);
    return list_id;
  };

  // List ID for an egress call.
  local_egress = function() {
    var caller, interesting_number, list, list_id, ref;
    caller = (ref = this.session.asserted) != null ? ref : this.source;
    list = `${caller}@${this.session.number_domain}`;
    interesting_number = this.destination;
    list_id = `list:${list}@${interesting_number}`;
    this.debug('local_egress', list, interesting_number);
    return list_id;
  };

  // Global ingress
  global_ingress = function() {
    var list_id;
    if (!((this.session.ccnq_to_164 != null) && (this.session.ccnq_from_164 != null))) {
      return false;
    }
    return list_id = `list:${this.session.ccnq_to_e164}@${this.session.ccnq_from_e164}`;
  };

  // Global egress
  global_egress = function() {
    var list_id;
    if (!((this.session.ccnq_to_164 != null) && (this.session.ccnq_from_164 != null))) {
      return false;
    }
    return list_id = `list:${this.session.ccnq_from_e164}@${this.session.ccnq_to_e164}`;
  };

  // List retriever
  // --------------
  get_list = async function(list_id) {
    var list;
    if (list_id) {
      list = (await this.cfg.prov.get(list_id).catch(function() {
        return {};
      }));
      if (list.disabled) {
        return null;
      } else {
        return list;
      }
    } else {
      return null;
    }
  };

  // List status
  // -----------
  is_blacklisted = function(list) {
    return list != null ? list.blacklist : void 0;
  };

  is_whitelisted = function(list) {
    return list != null ? list.whitelist : void 0;
  };

  is_suspicious = function(list) {
    return list != null ? list.suspicious : void 0;
  };

  // Commands builder
  // ----------------
  chain = function(test, selector) {
    return async function() {
      var list, list_id;
      list_id = (await selector.call(this));
      list = (await get_list.call(this, list_id));
      return test.call(this, list);
    };
  };

  // Commands
  // ========

  // * doc.number_domain.calendars: array or object of calendars. Each calendar is an array of dates in `YYYY-MM-DD` format. The calendars are used as filters by doc.local_number.ornaments' `in_calendars` command.
  menu_conference_module = {
    name: 'menu_conference_module',
    include: function() {
      var item, m, number, ref, ref1;
      if (this.session.dialplan !== 'centrex') {
        return;
      }
      if (!(m = this.destination.match(/^82(\d+)$/))) {
        return;
      }
      number = parseInt(m[1], 10);
      item = (ref = this.session.number_domain_data) != null ? (ref1 = ref.conferences) != null ? ref1[number] : void 0 : void 0;
      if (item != null) {
        if (item.short_name == null) {
          item.short_name = `conf-${number}`;
        }
        if (item.full_name == null) {
          item.full_name = `${this.session.number_domain}-${item.short_name}`;
        }
        this.session.conf = item;
        this.direction('conf');
      }
    }
  };

  ingress_modules = [];

  module.exports = commands = {
    // Actions
    // -------

    // These actions are terminal for the statement.
    stop: function() {
      this.debug('stop');
      return 'over';
    },
    accept: function() {
      this.debug('accept');
      return 'over';
    },
    hangup: async function() {
      this.debug('hangup');
      await this.action('hangup');
      this.direction('hangup');
      return 'over';
    },
    send: async function(destination) {
      this.debug('send');
      this.session.direction = 'ingress';
      this.destination = destination;
      await serialize.modules(ingress_modules, this, 'include');
      return 'over';
    },
    // `menu_send`: send the call to the (ingress) destination keyed (must be a number in the current number-domain)
    menu_send: async function() {
      this.debug('menu_send');
      if (this.menu == null) {
        return false;
      }
      await this.menu.expect();
      this.debug('menu_send', this.menu.value);
      this.session.direction = 'ingress';
      this.destination = this.menu.value;
      await serialize.modules([menu_conference_module, ...ingress_modules], this, 'include');
      return 'over';
    },
    reject: async function() {
      this.debug('reject');
      await this.respond('486 Decline');
      return 'over';
    },
    announce: async function(message) {
      this.debug('announce', message);
      await this.action('answer');
      await this.action('playback', `${this.cfg.provisioning}/config%3Avoice_prompts/${message}.wav`);
      await this.action('hangup');
      return 'over';
    },
    voicemail: function() {
      this.debug('voicemail');
      this.direction('voicemail');
      return 'over';
    },
    forward: function(destination) {
      this.debug('forward', destination);
      this.session.reason = 'unspecified';
      this.session.destination = destination;
      this.direction('forward');
      return 'over';
    },
    // Other actions must return `true`.
    email: function(recipient, template) {
      this.debug('email', recipient, template);
      // FIXME TODO
      return true;
    },
    // `play`: play a file, uninterrupted (should be used for short prompts)
    play: async function(file) {
      var url;
      this.debug('play', file);
      url = this.prompt.uri('prov', 'ignore', this.session.number_domain_data._id, file);
      await this.action('answer');
      await this.unset('playback_terminators');
      await this.action('playback', url);
      return true;
    },
    message: async function(number) {
      var file, ref, ref1, ref2, url;
      this.debug('message', number);
      if (!((ref = this.session.number_domain_data) != null ? (ref1 = ref.msg) != null ? (ref2 = ref1[number]) != null ? ref2.active : void 0 : void 0 : void 0)) {
        return true;
      }
      file = `msg-${number}.mp3`;
      url = this.prompt.uri('prov', 'ignore', this.session.number_domain_data._id, file);
      await this.action('answer');
      await this.unset('playback_terminators');
      await this.action('playback', url);
      return true;
    },
    // `menu_play`: play a file, stop playing when a key is pressed
    menu_play: async function(file) {
      var url;
      this.debug('menu_play', file);
      url = this.prompt.uri('prov', 'ignore', this.session.number_domain_data._id, file);
      await this.action('answer');
      await this.dtmf.playback(url);
      return true;
    },
    // `music`: set the music-on-hold
    music: function(file) {
      this.session.music = this.prompt.uri('prov', 'ignore', this.session.number_domain_data._id, file);
      return true;
    },
    // `ringback`: set the ringback
    ringback: function(file) {
      this.session.ringback = this.prompt.uri('prov', 'ignore', this.session.number_domain_data._id, file);
      return true;
    },
    wait: async function(ms) {
      this.debug('wait', ms);
      await this.dtmf.playback(`silence_stream://${ms}`);
      this.debug('wait over', ms);
      return true;
    },
    record: function(name) {
      return this.record_call(name);
    },
    // Preconditions
    // -------------

    // These are best used after `post` is used but before `send` is used.
    source: function(source) {
      this.debug('source', source);
      return pattern(source, this.source);
    },
    source_e164: function(source) {
      this.debug('source_e164', source);
      return pattern(source, this.session.ccnq_from_e164);
    },
    destination: function(destination) {
      this.debug('destination', destination);
      return pattern(destination, this.destination);
    },
    destination_e164: function(destination) {
      this.debug('destination_e164', destination);
      return pattern(destination, this.session.ccnq_to_e164);
    },
    // So, time conditions.
    // Maybe we first need to figure out in what timezone we are working.
    // Then get some Date object up and running.

    // Weekday condition
    weekdays: function(...days) {
      var now, ref;
      this.debug('weekdays', ...days);
      now = Moment();
      if (this.session.timezone != null) {
        now = now.tz(this.session.timezone);
      }
      return ref = now.day(), indexOf.call(days, ref) >= 0;
    },
    // Time condition
    time: function(start, end) {
      var now;
      this.debug('time', start, end);
      now = Moment();
      if (this.session.timezone != null) {
        now = now.tz(this.session.timezone);
      }
      now = now.format('HH:mm');
      // start: '09:00', end: '17:00'
      if (start <= end) {
        return (start <= now && now <= end);
      } else {
        // start: '18:00', end: '08:00'
        return start <= now || now <= end;
      }
    },
    user_tag: function(tag) {
      this.user_tag(tag);
      this.notify({
        state: 'menu',
        user_tag: tag
      });
      return true;
    },
    alert_info: async function(alert_info) {
      await this.export({alert_info});
      this.session.alert_info = alert_info;
      return true;
    },
    clear_call_center_tags: async function() {
      await this.clear_call_center_tags();
      return true;
    },
    clear_user_tags: async function() {
      await this.clear_user_tags();
      return true;
    },
    required_skill: async function(skill) {
      await this.tag(`skill:${skill}`);
      return true;
    },
    priority: async function(priority) {
      await this.tag(`priority:${priority}`);
      return true;
    },
    queue: async function(queue) {
      await this.tag(`queue:${queue}`);
      return true;
    },
    broadcast: async function() {
      await this.tag('broadcast');
      return true;
    },
    has_tag: function(tag) {
      return this.has_tag(tag);
    },
    has_user_tag: function(tag) {
      return this.has_user_tag(tag);
    },
    agent_skill: async function(skill) {
      if (typeof skill !== 'string') {
        return false;
      }
      await this.agent.add_tag(`skill:${skill}`);
      return true;
    },
    agent_queue: async function(queue) {
      if (typeof queue !== 'string') {
        return false;
      }
      await this.agent.add_tag(`queue:${queue}`);
      return true;
    },
    agent_broadcast: async function() {
      await this.agent.add_tag("broadcast");
      return true;
    },
    // Calendars
    in_calendars: function(...calendars) {
      var calendar, dates, domain_calendars, hd, i, len, m, now, now_date, ref, ref1, ref2, res;
      this.debug('calendars', calendars);
      // Legacy format: only one argument and that argument is an array.
      if (calendars.length === 1 && ((ref = typeof calendars[0]) !== 'string' && ref !== 'number')) {
        calendars = calendars[0];
      }
      domain_calendars = (ref1 = (ref2 = this.session.number_domain_data) != null ? ref2.calendars : void 0) != null ? ref1 : {};
      now = Moment();
      if (this.session.timezone != null) {
        now = now.tz(this.session.timezone);
      }
      now_date = new Date(now);
      now = now.format('YYYY-MM-DD');
      for (i = 0, len = calendars.length; i < len; i++) {
        calendar = calendars[i];
        switch (false) {
          case !(m = calendar.toString().match(/^_holidays_(.*)$/)):
            hd = new Holidays();
            hd.init.apply(hd, m[1].split('_'));
            res = hd.isHoliday(now_date);
            if ((res != null ? res.type : void 0) === 'public') {
              return true;
            }
            break;
          case calendar !== '_holidays':
            if (this.session.country) {
              hd = new Holidays(this.session.country.toUpperCase());
              res = hd.isHoliday(now_date);
              if ((res != null ? res.type : void 0) === 'public') {
                return true;
              }
            }
            break;
          default:
            if (domain_calendars[calendar] != null) {
              ({dates} = domain_calendars[calendar]);
              if ((dates != null) && indexOf.call(dates, now) >= 0) {
                return true;
              }
            }
        }
      }
      return false;
    },
    anonymous: function() {
      this.debug('anonymous');
      return this.session.caller_privacy;
    },
    webhook: async function(uri) {
      var body;
      this.debug('webhook');
      try {
        ({body} = (await request.post(uri).send({
          tags: (await this.reference.tags()),
          ccnq_from_e164: this.session.ccnq_from_e164,
          ccnq_to_e164: this.session.ccnq_to_e164,
          _in: this._in()
        })));
        if (body != null) {
          await this.user_tags(body.tags);
        }
        return true;
      } catch (error) {
        this.debug('webhook: error');
        return false;
      }
    },
    // Postconditions
    // --------------

    // These really only apply after the call has gone through `send` (but probably before it goes through `post-send`.)
    // They can be used to provide further call treatment, similar to the various `CF..` conditions.
    busy: function() {
      this.debug('busy');
      return this.session.reason === 'user-busy';
    },
    unavailable: function() {
      this.debug('unavailable');
      return this.session.reason === 'unavailable';
    },
    'no-answer': function() {
      this.debug('no-anwer');
      return this.session.reason === 'no-answer';
    },
    // Notice: `failed` here means the call failed to be sent to the user *and* no other CF.. condition handled it (sending to voicemail using `cfnr_voicemail` is not considered a failure for example).
    failed: function() {
      this.debug('failed');
      return this.session.call_failed;
    },
    answered: function() {
      this.debug('answered');
      if (this.session.was_connected) {
        return true;
      } else {
        return false;
      }
    },
    picked: function() {
      this.debug('picked');
      if (this.session.was_picked) {
        return true;
      } else {
        return false;
      }
    },
    transferred: function() {
      this.debug('transferred');
      if (this.session.was_transferred) {
        return true;
      } else {
        return false;
      }
    },
    caller_blacklist: chain(is_blacklisted, local_ingress),
    called_blacklist: chain(is_blacklisted, local_egress),
    caller_e164_blacklist: chain(is_blacklisted, global_ingress),
    called_e164_blacklist: chain(is_blacklisted, global_egress),
    caller_whitelist: chain(is_whitelisted, local_ingress),
    called_whitelist: chain(is_whitelisted, local_egress),
    caller_e164_whitelist: chain(is_whitelisted, global_ingress),
    called_e164_whitelist: chain(is_whitelisted, global_egress),
    caller_suspicious: chain(is_suspicious, local_ingress),
    called_suspicious: chain(is_suspicious, local_egress),
    caller_e164_suspicious: chain(is_suspicious, global_ingress),
    called_e164_suspicious: chain(is_suspicious, global_egress),
    // Menus
    // -----

    // `menu_start`: start collecting digits for a menu; digits received before this command are discarded.
    menu: async function(min = 1, max = min, itd) {
      this.debug('menu_start');
      await this.action('answer');
      this.dtmf.clear();
      this.menu = {
        expect: async() => {
          var base;
          return (base = this.menu).value != null ? base.value : base.value = (await this.dtmf.expect(min, max, itd));
        }
      };
      return true;
    },
    // `menu_on`: true if the user keyed the choice
    menu_on: async function(choice) {
      choice = `${choice}`;
      this.debug('menu_on', choice);
      if (this.menu == null) {
        return false;
      }
      await this.menu.expect();
      this.debug('menu_on', choice, this.menu.value);
      return this.menu.value === choice;
    },
    goto_menu: async function(number) {
      var item, items, type;
      this.debug('goto_menu', number);
      // Copying the logic from middleware/client/ingress/fifo
      type = 'menu';
      items = this.session.number_domain_data.menus;
      if (items == null) {
        this.debug.csr(`Number domain has no data for ${type}.`);
        return;
      }
      if (!items.hasOwnProperty(number)) {
        this.debug.dev(`No property ${number} in ${type} of ${this.session.number_domain}`);
        return;
      }
      item = items[number];
      if (item == null) {
        this.debug.csr(`Number domain as no data ${number} for ${type}.`);
        return;
      }
      this.session[type] = item;
      this.direction(type);
      this.debug(`Using ${type} ${number}`, item);
      if (this.menu_depth == null) {
        this.menu_depth = 0;
      }
      this.menu_depth++;
      if (this.menu_depth > max_menu_depth) {
        return false;
      }
      await this.sleep(200);
      await run.call(this, item, this.ornaments_commands);
      return 'over';
    }
  };

  // Pattern
  // -------

  // Does the number `n` match the pattern `p`.

  // The pattern must consists of only:
  // - digits
  // - '?', '.' -- replace single of above
  // - '..', '...', '…' -- replace zero or more
  pattern = function(p, n) {
    var r;
    if (!p.match(/^(\d|\?|\.|\.\.|\.\.\.|…)+$/)) {
      debug('invalid pattern', p);
      return false;
    }
    p = p.replace(/\.\.|\.\.\.|…/g, '\\d*').replace(/\?|\./g, '\\d');
    r = new RegExp(`^${p}$`);
    debug('pattern', p, r);
    return n.match(r);
  };

}).call(this);
