// Generated by CoffeeScript 2.3.0
(function() {
  var debug, pkg, qs, second, sleep,
    hasProp = {}.hasOwnProperty;

  pkg = require('../../package');

  this.name = `${pkg.name}:middleware:client:fifo`;

  debug = (require('tangible'))(this.name);

  qs = require('querystring');

  this.description = 'Handles routing to a given queue/ACD/hunt-group.';

  sleep = function(timeout) {
    return new Promise(function(resolve) {
      return setTimeout(resolve, timeout);
    });
  };

  second = 1000;

  this.include = async function() {
    var announce_uri, attempt_overflow, base, call, call_is_answered, call_tags, cause, causes, data, fifo, fifo_progress_timeout, fifo_timeout, fifo_uri, i, id, ingress_pool, j, k, l, leg_delay_start, leg_progress_timeout, leg_timeout, len, len1, len2, m, member, music_uri, queuer, recipient, recipients, ref, ref1, ref2, ref3, ref4, ref5, ref6, ref7, ref8, ref9, ref_tags, res, skill, sofias, v, xfer;
    // FIFO handling
    // =============
    this.debug('Starting');
    if (((ref = this.session) != null ? ref.direction : void 0) !== 'fifo') {
      return;
    }
    fifo_uri = (id, name) => {
      return this.prompt.uri('prov', 'prov', id, name);
    };
    if (this.session.fifo == null) {
      this.debug('Missing FIFO data');
      return;
    }
    fifo = this.session.fifo;
    // Build the full fifo name (used inside FreeSwitch) from the short fifo-name and the number-domain.
    this.notify({
      state: 'fifo',
      fifo: fifo.full_name
    });
    // Ready to send, answer the call.
    this.debug('Answer');
    await this.action('answer');
    call_is_answered = true;
    await this.export({
      t38_passthru: false,
      // Basically if the pre_answer we should wait; once the call is answered we won't be getting any more ACK, though.
      sip_wait_for_aleg_ack: !call_is_answered
    });
    // * session.fifo.announce (string) Name of the FIFO announce file (attachment to the doc:number_domain document).
    // * session.fifo.music (string) Name of the FIFO music file (attachment to the doc:number_domain document).
    id = `number_domain:${this.session.number_domain}`;
    this.debug('Send to FIFO');
    await this.set({
      continue_on_fail: true
    });
    if (this.session.ringback != null) {
      announce_uri = this.session.ringback;
    }
    if (this.session.announce != null) {
      announce_uri = this.session.announce;
    }
    if (fifo.announce != null) {
      announce_uri = fifo_uri(id, fifo.announce);
    }
    if (announce_uri != null) {
      await this.set({
        ringback: announce_uri
      });
    }
    if (this.session.music != null) {
      music_uri = this.session.music;
    }
    if (fifo.music != null) {
      music_uri = fifo_uri(id, fifo.music);
    }
    if (music_uri != null) {
      await this.export({
        hold_music: music_uri
      });
    }
    // FIXME: This is taken from the centrex-{country} code, but really it should be more generic.
    if (this.session.ccnq_from_e164 != null) {
      this.source = `+${this.session.ccnq_from_e164}`;
    }
    if (fifo.tags != null) {
      await this.user_tags(fifo.tags);
    }
    if (fifo.required_skills != null) {
      ref1 = fifo.required_skills;
      for (j = 0, len = ref1.length; j < len; j++) {
        skill = ref1[j];
        await this.tag(`skill:${skill}`);
      }
    }
    if (typeof fifo.queue === 'string') {
      await this.tag(`queue:${fifo.queue}`);
    }
    if (fifo.priority != null) {
      await this.tag(`priority:${fifo.priority}`);
    }
    if (fifo.broadcast) {
      await this.tag('broadcast');
    }
    // Call-center
    // ===========

    // If the call-group should use the queuer, then do that.
    if (fifo.queue) {
      this.notify({
        state: 'queue',
        name: fifo.full_name,
        queue: fifo.queue
      });
      ({queuer} = this.cfg);
      call = (await this.queuer_call());
      ref_tags = (await this.reference.tags());
      await call.set_tags(ref_tags);
      await call.add_tag(`number_domain:${this.session.number_domain}`);
      if (music_uri != null) {
        await call.set_music(music_uri);
      }
      if (announce_uri != null) {
        this.action('endless_playback', announce_uri); // async
      }
      await call.set_remote_number(this.source);
      if (this.session.alert_info != null) {
        await call.set_alert_info(this.session.alert_info);
      }
      await call.clear();
      await queuer.queue_ingress_call(call);
      // If the call is not processed (no agents are ready), attemp overflow.

      // Overflow is the weird concept that instead of giving a caller access to all of our capable
      // agents immediately, we decide to lengthen response time and leave some agents idle, by only
      // allowing access to them if some conditions are met.
      // Since we are trying to extend the pool of agents, this is only possible by adding more
      // desirable queues to a given call. (Adding more required skills would build a smaller pool.)
      call_tags = ref_tags.filter(function(tag) {
        return tag.match(/^queue:/);
      });
      if (call_tags.length === 0) {
        this.debug('no queues, no overflow');
        return;
      }
      ingress_pool = queuer.ingress_pool(this.session.number_domain);
      attempt_overflow = async(suffix) => {
        var l, len1, ok, tag;
        this.debug('attempt overflow', call_tags, suffix);
        await call.load();
        if ((await ingress_pool.has(call))) {
          ok = false;
          for (l = 0, len1 = call_tags.length; l < len1; l++) {
            tag = call_tags[l];
            if (!((await call.has_tag(tag)))) {
              continue;
            }
            await call.add_tag(`${tag}:${suffix}`);
            ok = true;
          }
          return ok;
        } else {
          return false;
        }
      };
      // Attempt overflow immediately
      if (!attempt_overflow('overflow')) {
        return;
      }
      // Attempt overflow after a delay
      await sleep(30 * second);
      if (!attempt_overflow('overflow:30s')) {
        return;
      }
      await sleep(30 * second);
      if (!attempt_overflow('overflow:60s')) {
        return;
      }
      return;
    }
    // Hunt-group
    // ==========

    // Otherwise use the hunt-group behavior.
    this.notify({
      state: 'group',
      name: fifo.full_name
    });
    sofias = [];
    // * session.fifo.members (array) (required) List of static members for this hunt-group/ACD/FIFO.
    // * session.fifo.members[].recipient (string) (required) Local-number in the FIFO's number-domain. The recipients are dialed without regards to their CFA, .. settings.
    // * session.fifo.members[].delay (integer) Number of seconds to wait before trying to call this recipient, in seconds. Zero or `null` means 'call immediately'. Default: 0.
    // * session.fifo.members[].progress_timeout (integer) Number of seconds before declaring a recipient unreachable (unable to ring the phone). Default: the progress-timeout of the FIFO.
    // * session.fifo.members[].timeout (integer) Number of seconds before declaring a recipient unreachable (did not answer), in seconds. Zero means 'wait indefinitely'. Default: the timeout of the FIFO.
    // * session.fifo.timeout (integer) Default number of seconds before declaring a recipient unreachable (how long to let the recipient's phone ring). Default: zero, meaning 'wait indefinitely'.
    // * session.fifo.progress_timeout (integer) Default number of seconds before declaring a recipient unreachable (unable to ring the phone). Default: 4.
    fifo_timeout = (ref2 = fifo.timeout) != null ? ref2 : 0;
    fifo_progress_timeout = (ref3 = fifo.progress_timeout) != null ? ref3 : 4;
    recipients = [];
    ref4 = fifo.members;
    for (i = l = 0, len1 = ref4.length; l < len1; i = ++l) {
      member = ref4[i];
      // Backward-compatible
      if ('string' === typeof member) {
        recipient = member;
        leg_delay_start = i * 5;
        leg_progress_timeout = 4;
        leg_timeout = 60;
      } else {
        recipient = member.recipient;
        leg_delay_start = (ref5 = member.delay) != null ? ref5 : 0;
        leg_progress_timeout = (ref6 = member.progress_timeout) != null ? ref6 : fifo_progress_timeout;
        leg_timeout = (ref7 = member.timeout) != null ? ref7 : fifo_timeout;
      }
      recipients.push(`${recipient}@${this.session.number_domain}`);
      sofias.push((await this.sofia_string(recipient, [
        (function() {
          var ref8,
        results;
          ref8 = {
            t38_passthru: false,
            leg_delay_start,
            leg_progress_timeout,
            leg_timeout
          };
          results = [];
          for (k in ref8) {
            if (!hasProp.call(ref8, k)) continue;
            v = ref8[k];
            results.push(`${k}=${v}`);
          }
          return results;
        })()
      ])));
    }
    this.debug('bridge', sofias);
    res = (await this.action('bridge', sofias.join(',')));
    data = res.body;
    if ((base = this.session).bridge_data == null) {
      base.bridge_data = [];
    }
    this.session.bridge_data.push(data);
    this.debug('Returned from FIFO', data);
    // Marking missed calls
    // --------------------

    // `variable_originated_legs: [ 'uuid:name:number', … ]`
    // `variable_originate_causes: [ 'uuid:cause', … ]`

    // Some causes:
    // - `NONE` -- race winner
    // - `LOSE_RACE` -- race loser
    // - `ALLOTTED_TIMEOUT` -- all losers
    // - `NO_ROUTE_DESTINATION` -- technical difficulty
    // - `ORIGINATOR_CANCEL` -- caller hung up, all losers

    // FIXME: what is the `cause` when a call was not presented? (esp. if there was a winner)
    causes = (ref8 = data.variable_originate_causes) != null ? ref8.map(function(str) {
      return str.split(':')[1];
    }) : void 0;
    ref9 = causes != null ? causes : [];
    for (i = m = 0, len2 = ref9.length; m < len2; i = ++m) {
      cause = ref9[i];
      ((cause, agent) => {
        switch (cause) {
          case 'NONE':
            this.notify({
              state: 'hunt-bridging',
              agent
            });
            return this.notify({
              state: 'answered',
              agent
            });
          case 'NO_ROUTE_DESTINATION':
            return false;
          default:
            this.notify({
              state: 'hunt-bridging',
              agent
            });
            return this.notify({
              event: 'missed',
              agent
            });
        }
      })(cause, recipient[i]);
    }
    // Detecting transfer
    // ------------------

    // Available parameters related to transfer are (in the case `bridge` is used):
    // - `variable_transfer_source` (string)
    // - `Caller-Transfer-Source` (string)
    // - `variable_transfer_history` (string or array)

    // Blind-transfer:
    // - `variable_transfer_history`: `"1461931135:41606813-d6c0-4bf9-af49-b2017c23ab7c:bl_xfer:endless_playback:http://(nohead=true)127.0.0.1:5714/fifo/number_domain%3Atest.centrex.phone..."`

    // Attended-transfer:
    // - `variable_transfer_history`: `[ "1461928571:4edd3fa0-8725-41a4-9322-d1e6b636a913:att_xfer:10@test.centrex.phone../33643482771@178.250.209.67", "1461928571:017ae036-cfdb-4e35-9bd8-0686db3897a0:uuid_br:36093e6f-cc71-484e-8cc7-eb70d42c10be" ]`

    // The first number in those strings is the timestamp in seconds.
    // In the case of `uuid_br`, the UUID at the end is the `Other-Leg-Unique-ID`.
    xfer = data.variable_transfer_history;
    if (xfer != null) {
      this.debug('Call was transferred', xfer);
      return;
    }
    cause = data.variable_originate_disposition;
    this.debug(`FIFO returned with cause ${cause}`);
    if (cause === 'NORMAL_CALL_CLEARING' || cause === 'SUCCESS' || cause === 'NORMAL_CLEARING') {
      this.debug(`Successful call when routing FIFO ${fifo.full_name} through ${sofias.join(',')}`);
      await this.action('hangup');
      return;
    }
    if (cause === 'ORIGINATOR_CANCEL') {
      return;
    }
    // * session.fifo.voicemail (string) If present, the call is redirected to this number's voicemail box if the FIFO failed (for example because no agents are available).
    // * session.fifo.user_database (string) If present, the call is redirected to this voicemail box if the FIFO failed (for example because no agents are available). Default: use session.fifo.voicemail if present.
    if (fifo.voicemail != null) {
      this.debug('Send to voicemail');
      this.destination = fifo.voicemail;
      this.direction('voicemail');
      await this.validate_local_number();
      return;
    }
    if (fifo.user_database != null) {
      this.debug('Send to voicemail (user-database)');
      this.destination = 'user-database';
      this.session.voicemail_user_database = fifo.user_database;
      this.session.voicemail_user_id = fifo.full_name;
      this.direction('voicemail');
      return;
    }
    this.debug('Hangup');
    return (await this.action('hangup'));
  };

}).call(this);
