// Generated by CoffeeScript 2.3.0
(function() {
  var debug, pkg;

  pkg = require('../../../package.json');

  this.name = `${pkg.name}:middleware:client:ingress:national-FR`;

  debug = (require('tangible'))(this.name);

  this.include = function() {
    var $, ref;
    if (((ref = this.session) != null ? ref.direction : void 0) !== 'ingress') {
      return;
    }
    if (this.session.dialplan !== 'e164') {
      return;
    }
    debug('Ready', {
      destination: this.destination,
      source: this.source
    });
    switch (false) {
      // Rewrite destination
      // ===================

      // * session.ccnq_to_e164 (string) The original destination number (in E.164-sans-plus format) before translation to a national number.
      // * session.country (string) Two-letter name of the country in which a national number must be interpreted. Typically used when `session.dialplan` equals `national`.
      case !($ = this.destination.match(/^33([1-9][0-9]+)$/)):
        this.session.ccnq_to_e164 = this.destination;
        this.destination = `0${$[1]}`;
        this.session.country = 'fr';
        break;
      default:
        return;
    }
    switch (false) {
      // from: national number

      // ARCEP, décision 2012-0856 au VI.1
      // Rewrite source
      // ==============

      // * session.ccnq_from_e164 (string) The original calling number (in E.164-sans-plus format) before translation to a national number.
      // Destination _must_ be a France number.
      // Otherwise let it be parsed by another module.
      case !($ = this.source.match(/^3389|^089/)):
        debug('Calling number is blocked per ARCEP 05-1085 2.b.1.iii page 14');
        this.direction('trash');
        return this.respond('484');
      case !($ = this.source.match(/^33([0-9]+)$/)):
        this.session.ccnq_from_e164 = this.source;
        this.source = `0${$[1]}`;
        break;
      // from: international number
      case !($ = this.source.match(/^([1-9][0-9]+)$/)):
        this.session.ccnq_from_e164 = this.source;
        this.source = `00${$[1]}`;
        break;
      // from: anonymous
      case this.source !== 'anonymous':
        debug('Source is anonymous');
        break;
      default:
        debug('Invalid source', this.source);
        return this.respond('484 Invalid source');
    }
    // Update the dialplan
    // ===================

    // * session.dialplan (string) The dialplan in which a number (source, destination) must be interpreted.
    this.session.dialplan = 'national';
    debug('OK', {
      source: this.source,
      destination: this.destination,
      diaplan: this.session.dialplan,
      country: this.session.country
    });
  };

}).call(this);
