// Generated by CoffeeScript 2.3.0
(function() {
  // This module should be called before 'local/carrier-ingress' and before 'client-sbc/$${profile_type}-ingress'
  var debug, pkg;

  pkg = require('../../../package.json');

  this.name = `${pkg.name}:middleware:client:ingress:pre`;

  debug = (require('tangible'))(this.name);

  this.include = async function() {
    var ref;
    if (((ref = this.session) != null ? ref.direction : void 0) !== 'ingress') {
      return;
    }
    await this.reference.set_endpoint(null);
    // Do not process here if the dialplan is already known (e.g. because Centrex sent us here).
    if (this.session.dialplan != null) {
      this.debug('Dialplan already set, skipping');
      return;
    }
    this.debug('Ready');
    // Global number
    // -------------

    // We retrieve the *global-number* record based on the destination.

    // * session.e164_number (object) The doc.global_number record for of the destination of an inbound call.
    this.session.e164_number = (await this.cfg.prov.get(`number:${this.destination}`).catch(function() {
      return null;
    }));
    if (this.session.e164_number == null) {
      return;
    }
    // E.164
    // -----

    // All external ingress calls come in as E.164 (without plus sign).
    this.session.dialplan = 'e164';
    this.session.ccnq_from_e164 = this.source;
    this.session.ccnq_to_e164 = this.destination;
    await this.reference.add_in(this.session.e164_number._id);
    await this.user_tags(this.session.e164_number.tags);
    // The global number might contain additional FreeSwitch variables. Load these extra variables from the record.

    // * session.e164_number.fs_variables See doc.global_number.fs_variables
    // * doc.global_number (object, optional) Additional FreeSwitch variables to be set on ingress calls (for the called number). These will show up in CDRs on the client side.
    if (this.session.e164_number.fs_variables != null) {
      this.debug('Using fs_variables');
      await this.set(this.session.e164_number.fs_variables);
    }
    // * doc.global_number.trace (boolean) trace
    if (this.session.e164_number.trace) {
      this.session.dev_logger = true;
    }
    // * session.e164_number.voicemail_main See doc.global_number.voicemail_main
    // * doc.global_number.voicemail_main (boolean) If true, the number is the main number for access to voicemail (from an external number).
    // * session.e164_number.language See doc.global_number.language
    // * doc.global_number.language (string) Language-code to use for features, e.g. voicemail.
    if (this.session.e164_number.voicemail_main) {
      this.debug('Using voicemail_main');
      this.destination = 'main';
      this.session.language = this.session.e164_number.language;
      // * doc.global_number.voicemail_number_domain (string) the number-domain for access to voicemail (from an external number).
      this.session.number_domain = this.session.e164_number.voicemail_number_domain;
      this.direction('voicemail');
    }
    this.notify({
      state: 'ingress',
      e164_number: this.session.e164_number._id
    });
    this.debug('OK');
  };

}).call(this);
