// Generated by CoffeeScript 2.3.0
(function() {
  // Auto-call
  // ---------

  // This module places calls (presumably towards a client or Centrex extension) and sends them into the socket we control.
  // It ensures data is retrieved and injected in the call.

  // This module also triggers calls from within a conference.
  var Moment, UPDATE, debug, escape, foot, handler, make_params, now, pkg,
    hasProp = {}.hasOwnProperty;

  pkg = require('../../package');

  this.name = `${pkg.name}:middleware:client:place-call`;

  ({debug, foot} = (require('tangible'))(this.name));

  Moment = require('moment-timezone');

  ({UPDATE} = require('red-rings/operations'));

  escape = function(v) {
    return `${v}`.replace(',', ',');
  };

  make_params = function(data) {
    var k, v;
    return ((function() {
      var results;
      results = [];
      for (k in data) {
        if (!hasProp.call(data, k)) continue;
        v = data[k];
        results.push(`${k}=${escape(v)}`);
      }
      return results;
    })()).join(',');
  };

  now = function(tz = 'UTC') {
    return Moment().tz(tz).format();
  };

  this.handler = handler = function(cfg) {
    var Reference, call_to_conference, client_server, context, create_queuer_call, elected, host, local_server, p, place_call, port, profile, ref, ref1, ref2, ref3, ref4, ref5, socket_port, sofia_profile;
    if (cfg.local_redis_client != null) {
      elected = async function(key) {
        var name, winner;
        name = `elected-${key}`;
        winner = (await cfg.local_redis_client.setnx(name, false).catch(function() {
          return null;
        }));
        if (winner) {
          await cfg.local_redis_client.expire(name, 60).catch(function() {
            return true;
          });
        } else {
          debug('Lost the election.');
        }
        return winner;
      };
    } else {
      elected = function() {
        return Promise.resolve(true);
      };
    }
    // * cfg.session.profile (string) Configuration profile that should be used to place calls towards client, for automated calls.
    profile = (ref = cfg.session) != null ? ref.profile : void 0;
    if (profile == null) {
      debug.dev('Missing cfg.session.profile, not starting.');
      return;
    }
    sofia_profile = `huge-play-${profile}-egress`;
    context = `${profile}-egress`;
    // See huge-play/conf/freeswitch
    host = cfg.host;
    p = (ref1 = cfg.profiles) != null ? ref1[profile] : void 0;
    if (host && (p != null)) {
      port = (ref2 = p.egress_sip_port) != null ? ref2 : p.sip_port + 10000;
      socket_port = (ref3 = p.socket_port) != null ? ref3 : 5721;
    } else {
      debug('Missing cfg.host or cfg.profiles[profile], not starting.', {profile});
      return;
    }
    // Note: if there are multiple profiles in use we will get in trouble at that point. (FIXME)
    local_server = `${cfg.host}:${(ref4 = p.ingress_sip_port) != null ? ref4 : p.sip_port}`;
    client_server = `${cfg.host}:${(ref5 = p.egress_sip_port) != null ? ref5 : p.sip_port + 10000}`;
    debug('Using', {local_server, client_server});
    ({Reference} = cfg);
    // Place Call
    // ----------

    // The calls are automatically sent back to ourselves so that they can be processed like regular outbound calls.

    // This feature will call an extension (client-side number) and when the extension picks the call up, it will call the destination number (either an internal or external destination).

    // Event parameters:
    // - `_id` (YYYY-MM-UUID)
    // - `endpoint`
    // - `caller` (with appropriate `endpoint_via` translations if necessary)
    // - `destination`
    // - `callee_name` (optional)
    // - `callee_num` (optional)
    // - `call_timeout` (optional)
    place_call = foot(async function(data) {
      var _id, account, argv, caller, cmd, command, domain, endpoint, endpoint_data, is_remote, my_reference, params, ref6, res, sofia, timezone, xref;
      ({endpoint, caller, _id} = data);
      if (data.callee_name == null) {
        data.callee_name = pkg.name;
      }
      if (data.callee_num == null) {
        data.callee_num = data.destination;
      }
      // FIXME The data sender must do resolution of the `endpoint_via` and associated translations????
      // ANSWER: Yes. And store the result in the field `caller`.
      debug('Received place-call', data);
      // A proper reference is required.
      if (!((_id != null) && _id.match(/^[\w-]+$/))) {
        return;
      }
      // Load additional data from the endpoint.
      endpoint_data = (await cfg.prov.get(`endpoint:${endpoint}`).catch(function() {
        return null;
      }));
      if (endpoint_data == null) {
        return;
      }
      if (endpoint_data.disabled || endpoint_data.src_disabled) {
        return;
      }
      ({account, timezone} = endpoint_data);
      // Ensure only one FreeSwitch server processes those.
      domain = endpoint_data.number_domain;
      if (domain == null) {
        return;
      }
      // Note that Centrex-redirect uses both the local-server and the client-server.
      is_remote = (await cfg.is_remote(domain, [local_server, client_server].join('/')).catch(function() {
        return true;
      }));
      if (is_remote) {
        return;
      }
      debug('place-call: Placing call');
      // Session Reference Data
      my_reference = new Reference(_id);
      await my_reference.add_in([`endpoint:${endpoint}`, `account:${account}`, `number_domain:${domain}`]);
      await my_reference.set_account(account);
      await my_reference.set_destination(data.destination);
      xref = `xref=${_id}`;
      params = make_params({
        // These are used by `huge-play/middleware/client/setup`.
        session_reference: _id,
        origination_context: context,
        sip_invite_params: xref,
        sip_invite_to_params: xref,
        sip_invite_contact_params: xref,
        sip_invite_from_params: xref,
        // And `ccnq4-opensips` requires `X-En` for routing.
        'sip_h_X-En': data.endpoint
      });
      sofia = `{${params}}sofia/${sofia_profile}/sip:${caller}`;
      command = `socket(127.0.0.1:${socket_port} async full)`;
      argv = [
        sofia,
        `'&${command}'`,
        // dialplan
        'none',
        // context
        context,
        // cid_name -- callee_name, shows on the caller's phone and in Channel-(Orig-)Callee-ID-Name
        data.callee_name,
        // cid_num -- called_num, shows on the caller's phone and in Channel-(Orig-)Callee-ID-Number
        data.callee_num,
        // timeout_sec
        (ref6 = data.call_timeout) != null ? ref6 : ''
      ].join(' ');
      cmd = `originate ${argv}`;
      if (!(await elected(_id))) {
        return;
      }
      debug(`Calling ${cmd}`);
      res = (await cfg.api(cmd).catch(function(error) {
        var msg, ref7;
        msg = (ref7 = error.stack) != null ? ref7 : error.toString();
        debug(`originate: ${msg}`);
        return msg;
      }));
      // The `originate` command will return when the call is answered by the callee (or an error occurred).
      debug("Originate returned", res);
      if (res[0] === '+') {
        debug('caller-connected', _id);
      } else {
        debug.dev('caller-failed', _id);
      }
      return debug('Session state:', data.tags);
    });
    // Call to conference
    // ------------------

    // Parameters:
    // - `_id` (YYYY-MM-UUID)
    // - `endpoint`
    // - `conference` (was `name`)
    // - `destination`
    call_to_conference = foot(async function(data) {
      var _id, account, calling_number, cmd, destination, endpoint, endpoint_data, is_remote, language, my_reference, name, number_data, params, ref6, ref7, res, sofia, timezone, xref;
      ({endpoint, destination, _id} = data);
      debug('Received call-to-conference', data, local_server);
      name = (ref6 = data.conference) != null ? ref6 : data.name;
      // A proper reference is required.
      if (!((_id != null) && _id.match(/^[\w-]+$/))) {
        return;
      }
      // Ensure we are co-located with the FreeSwitch instance serving this conference.
      is_remote = (await cfg.is_remote(name, local_server).catch(function() {
        return true;
      }));
      if (is_remote) {
        return;
      }
      // Load additional data from the endpoint.
      endpoint_data = (await cfg.prov.get(`endpoint:${endpoint}`).catch(function() {
        return null;
      }));
      if (endpoint_data == null) {
        return;
      }
      if (endpoint_data.disabled || endpoint_data.src_disabled) {
        return;
      }
      ({account, timezone} = endpoint_data);
      // Try to get the asserted number, assuming Centrex.
      number_data = (await cfg.prov.get(`number:${endpoint}`).catch(function() {
        return {};
      }));
      calling_number = (ref7 = number_data.asserted_number) != null ? ref7 : endpoint.split('@')[0];
      ({language} = data);
      if (language == null) {
        language = number_data.language;
      }
      if (language == null) {
        language = endpoint_data.language;
      }
      if (data.timezone != null) {
        if (timezone == null) {
          timezone = data.timezone;
        }
      }
      // Duplicated from exultant-song (FIXME)
      debug('call-to-conference: Placing call');
      // Session Reference Data
      my_reference = new Reference(_id);
      await my_reference.add_in([`endpoint:${endpoint}`, `account:${account}`, `number_domain:${endpoint.number_domain}`]);
      await my_reference.set_account(account);
      await my_reference.set_endpoint(endpoint);
      await my_reference.set_call_options({
        group_confirm_key: '5', // if `exec`, `file` holds the application and parameters; otherwise, one or more chars to confirm
        group_confirm_file: 'phrase:conference:confirm:5', // defaults to `silence`
        group_confirm_error_file: 'phrase:conference:confirm:5',
        group_confirm_read_timeout: 15000, // defaults to 5000
        group_confirm_cancel_timeout: false,
        language: language
      });
      // Call it out
      xref = `xref:${_id}`;
      params = make_params({
        sip_invite_params: xref,
        sip_invite_to_params: xref,
        sip_invite_contact_params: xref,
        sip_invite_from_params: xref,
        origination_caller_id_number: calling_number
      });
      sofia = `{${params}}sofia/${sofia_profile}/sip:${destination}@${host}:${port}`;
      cmd = `originate ${sofia} &conference(${name}++flags{})`;
      if (!(await elected(_id))) {
        return;
      }
      debug(`Calling ${cmd}`);
      res = (await cfg.api(cmd).catch(function(error) {
        var msg, ref8;
        msg = (ref8 = error.stack) != null ? ref8 : error.toString();
        debug(`conference: ${msg}`);
        return msg;
      }));
      return debug("Conference returned", res);
    });
    // Queuer place call
    // -----------------

    // The `body` should contains:
    // - `_id` (unique id for the request)
    // - `agent` (string)
    // - `destination` (string)
    // - `tags` (array)
    create_queuer_call = foot(async function(body) {
      var Agent, agent, is_remote, queuer;
      ({queuer} = cfg);
      ({Agent} = queuer);
      if (!((queuer != null) && (Agent != null))) {
        debug('create-queuer-call: no queuer');
        return;
      }
      if (!((body != null) && (body.agent != null) && (body.destination != null))) {
        debug('create-queuer-call: invalid content', body);
        return;
      }
      agent = new Agent(queuer, body.agent);
      is_remote = (await cfg.is_remote(agent.domain, [local_server, client_server].join('/')).catch(function() {
        return true;
      }));
      if (is_remote) {
        debug('create-queuer-call: not handled on this server', body);
        return;
      }
      if (!(await elected(body._id))) {
        return;
      }
      await queuer.create_egress_call_for(agent, body);
    });
    return {place_call, call_to_conference, create_queuer_call};
  };

  this.server_pre = function() {
    var H;
    H = handler(this.cfg);
    if (H == null) {
      return;
    }
    this.cfg.rr.receive('call:*', this.most_shutdown).filter(function({op, doc, deleted}) {
      return op === UPDATE && (doc != null) && !deleted;
    }).forEach(function(msg) {
      switch (false) {
        // `create_queuer_call`
        // - `_id` (unique id for the request)
        // - `agent` (string)
        // - `destination` (string)
        // - `tags` (array)
        case msg.doc.agent == null:
          return H.create_queuer_call(msg.doc);
        // `call_to_conference`
        // - `_id` (YYYY-MM-UUID)
        // - `endpoint`
        // - `conference` (was `name`)
        // - `destination`
        case msg.doc.conference == null:
          return H.call_to_conference(msg.doc);
        // `place_call`
        // - `_id` (YYYY-MM-UUID)
        // - `endpoint`
        // - 'caller' (with appropriate `endpoint_via` translations if necessary)
        // - `destination`
        // - `callee_name` (optional)
        // - `callee_num` (optional)
        // - `call_timeout` (optional)
        case msg.doc.endpoint == null:
          return H.place_call(msg.doc);
      }
    });
  };

  // Notify
  // ======

  // DEPRECATED
  this.notify = function({cfg, socket}) {
    var H;
    H = handler(cfg);
    if (H == null) {
      return;
    }
    socket.on('place-call', H.place_call);
    socket.on('call-to-conference', H.call_to_conference);
    socket.on('create-queuer-call', H.create_queuer_call);
    this.register('place-call', 'dial_calls');
    this.register('call-to-conference', 'dial_calls');
    this.register('create-queuer-call', 'dial_calls');
    this.configure({
      dial_calls: true
    });
    return debug('Module Ready');
  };

  // /DEPRECATED

  // Click-to-dial (`place-call`)
  // ----------------------------
  this.include = async function() {
    var account, destination, options, ref;
    // Force the destination for `place-call` calls (`originate` sets `Channel-Destination-Number` to the value of `Channel-Caller-ID-Number`).
    destination = (await this.reference.get_destination());
    if (destination != null) {
      this.destination = destination;
      await this.reference.set_destination(null);
      // Also, do not wait for an ACK, since we're calling out (to the "caller"),
      // and therefor the call is already connected by the time we get here.
      this.session.wait_for_aleg_ack = false; // in huge-play
      this.session.sip_wait_for_aleg_ack = false; // in tough-rate
      
      // Finally, generate a P-Charge-Info header so that the SBCs will allow the call through.
      account = this.reference.get_account();
      if (account != null) {
        await this.export({
          'sip_h_P-Charge-Info': `sip:${account}@${this.cfg.host}`
        });
      }
      this.set({
        ringback: (ref = this.session.ringback) != null ? ref : '%(3000,3000,437,1317)',
        instant_ringback: true
      });
      // "Tonalité d'acheminement", for nostalgia's sake.
      this.action('playback', 'tone_stream://%(50,50,437,1317);loops=-1');
    }
    // Options might be provided for either `place-call` or `call-to-conference`.
    // They are used in `tough-rate/middleware/call-handler`.
    options = (await this.reference.get_call_options());
    if (options != null) {
      this.session.call_options = options;
    }
    return this.debug('Ready', {destination, account, options});
  };

}).call(this);
